<?php
/**
 * This is an example of how to create a custom interface for Greyd Forms.
 */
if ( !defined( 'ABSPATH' ) ) {
	exit;
}

if ( !function_exists( 'register_my_custom_greyd_forms_interface' ) ) {

	/**
	 * Register the custom interface.
	 */
	function register_my_custom_greyd_forms_interface() {

		add_filter( 'handle_after_doi_'.GREYD_FORMS_CUSTOM_INTERFACE_NAME, 'handle_my_custom_greyd_forms_interface', 10, 4 );
	}
}
add_action( 'init', 'register_my_custom_greyd_forms_interface' );

if ( !function_exists( 'render_my_custom_interface_apikey' ) ) {

	/**
	 * Render Setting: My Custom Interface API key
	 *
	 * @param string $pre   The prefix for the setting.
	 * @param string $value The current value of the setting.
	 *
	 * @action render_setting_{{interface}}_{{option}}
	 */
	function render_my_custom_interface_apikey( $pre = '', $value = null ) {

		$option = 'apikey';
		$slug   = $pre.'['.$option.']';
		$value  = isset( $value ) ? $value : esc_attr( get_option( $slug, '' ) );

		echo "<input type='text' id='$slug' class='regular-text' name='$slug' value='$value'>";
	}
}
add_action( 'render_setting_'.GREYD_FORMS_CUSTOM_INTERFACE_NAME.'_apikey', 'render_my_custom_interface_apikey', 10, 2 );

if ( !function_exists( 'handle_my_custom_greyd_forms_interface' ) ) {

	/**
	 * After a form which uses this interface has been submitted, this function is called.
	 *
	 * @filter handle_after_doi_{{interface}}
	 *
	 * @param mixed  $response      The response to return.
	 * @param string $entry_id      The Post ID of the entry.
	 * @param array  $form_data     The user data, keyed by the field name.
	 * @param array  $post_meta     The form post_meta for this specific interface.
	 *
	 * @return bool True if submission was successful.
	 */
	function handle_my_custom_greyd_forms_interface( $response, $entry_id, $form_data, $post_meta ) {

		$api_key = \Greyd\Forms\Interfaces\Greyd_Forms_Interfaces::get_interface_settings(
			GREYD_FORMS_CUSTOM_INTERFACE_NAME,
			'apikey'
		);

		if ( empty( $api_key ) ) {
			/**
			 * Use the action 'formhandler_error' to display an error message to the user in
			 * the frontend. This is useful for displaying custom error messages. Otherwise
			 * just return false to trigger the default error handling.
			 */
			do_action( 'formhandler_error', 'No API key was provided.' );
		}

		// build the request body
		$request_body = array();

		// get static meta options
		$meta_options = isset( $post_meta['meta'] ) ? (array)$post_meta['meta'] : array();
		foreach ( $meta_options as $key => $val ) {
			if ( !empty( $val ) ) {
				$request_body[ $key ] = $val;
			}
		}

		// get default field assigments
		$default_fields = isset( $post_meta['normal'] ) ? (array)$post_meta['normal'] : array();
		foreach ( $default_fields as $key => $val ) {
			$value = isset( $form_data[ $val ] ) ? html_entity_decode( $form_data[ $val ] ) : '';

			// convert checkbox value to bool
			$value = strpos( $value, '[on]' ) !== false ? true : ( strpos( $value, '[off]' ) !== false ? false : $value );

			if ( $value !== '' ) {
				$request_body[ $key ] = $value;
			}
		}

		// get custom field assigments
		$custom_fields = isset( $post_meta['custom'] ) ? (array)$post_meta['custom'] : array();
		foreach ( $custom_fields as $key => $val ) {
			$value = isset( $form_data[ $val ] ) ? html_entity_decode( $form_data[ $val ] ) : '';

			// convert checkbox value to bool
			$value = strpos( $value, '[on]' ) !== false ? true : ( strpos( $value, '[off]' ) !== false ? false : $value );

			if ( $value !== '' ) {
				$request_body[ $key ] = $value;
			}
		}

		/**
		 * Use the action 'formhandler_log' to log something to the console log.
		 * This is useful for debugging purposes.
		 */
		do_action( 'formhandler_log', $request_body, 'Interface data' );

		/**
		 * By simply passing the $request_body array to the action 'formhandler_error',
		 * the data will be displayed in the frontend as a JSON string.
		 */
		// do_action( 'formhandler_error', $request_body );

		// Send the data to an API
		$response = wp_remote_post(
			'https://my-custom-api-endpoint.com',
			array(
				'method'  => 'POST',
				'headers' => array(
					'Content-Type'  => 'application/json',
					'Accept'        => '*/*',
					'Authorization' => 'Bearer '.$api_key,
				),
				'body'    => json_encode( $request_body ),
			)
		);
		// error_log( print_r( $response, true ) );

		/**
		 * Use the action 'formhandler_log' to log something to the console log.
		 * This is useful for debugging purposes.
		 */
		do_action( 'formhandler_log', $response, 'API response' );

		if ( is_wp_error( $response ) ) {
			\Greyd\Forms\Helper::log_entry_state(
				$entry_id,
				'Failed to send the date to the API.'.' ('.$response->get_error_message().')',
				'error',
				true // major entry state (visible in edit.php)
			);

			/**
			 * Use the action 'formhandler_error' to display an error message to the user in
			 * the frontend. This is useful for displaying custom error messages. Otherwise
			 * just return false to trigger the default error handling, which will not display
			 * any error message to the user but log the error to the entry and send an email
			 * to the admin.
			 */
			do_action( 'formhandler_error', 'An error occurred while sending the data to the API: '.$response->get_error_message() );

			return false;
		}

		// logging the success message to the entry
		\Greyd\Forms\Helper::log_entry_state(
			$entry_id,
			'Successfully sent the date to the API.',
			'success',
			true // major entry state (visible in edit.php)
		);

		/**
		 * Use the action 'formhandler_success' to display a success message to the user in
		 * the frontend. Otherwise just return true to trigger the default success handling.
		 *
		 * @param array $args An array of arguments to pass to the action. Currently supports:
		 *                 - post_id: The WP_Post ID of the form being submitted.
		 *                 - message: The success message to display.
		 */
		// do_action( 'formhandler_success', array(
		// 'post_id' => get_post_meta( $entry_id, 'tp_form_id', true ),
		// 'message' => 'Thank you for your submission. Your data has been sent to the API.',
		// ) );

		return true;
	}
}
