<?php
/**
 * Example of how to use different hooks with the Greyd.Forms plugin
 */

/**
 * Allow certain custom fields for form submissions.
 * Eg. Allow the custom field 'my_custom_field' for a specific form.
 * 
 * @param array $fields   The allowed fields, keyed by field name.
 * @param int   $form_id  The WP_Post ID of the form being submitted.
 * 
 * @return array The allowed fields, keyed by field name.
 */
function greyd_forms_allowed_custom_fields_example( $fields, $form_id ) {

	// Get the form post object
	$form = get_post( $form_id );

	// Check if the form is the one we want to allow the custom field for
	if ( $form && $form->post_name === 'my-contact-form' ) {

		// Add the custom field 'my_custom_field' to the allowed fields
		$fields['my_custom_field'] = true;
	}

	return $fields;
}
add_filter( 'formhandler_allow_fields', 'greyd_forms_allowed_custom_fields_example', 10, 2 );

/**
 * Filter the required fields for form submissions.
 * Eg. Add the custom field 'my_custom_field' to the required fields for a specific form.
 *
 * @param array  $required_fields   Array of required field names.
 * @param int    $form_id           The WP_Post ID of the form being submitted.
 * @param array  $form_data         Validated form data.
 */
function greyd_forms_required_fields_example( $required_fields, $form_id, $form_data ) {

	// Get the form post object
	$form = get_post( $form_id );

	// Check if the form is the one we want to allow the custom field for
	if ( $form && $form->post_name === 'my-contact-form' ) {
		
		// Add the custom field 'my_custom_field' to the required fields
		$required_fields[] = 'my_custom_field';
	}

	return $required_fields;
}
add_filter( 'greyd_forms_required_fields', 'greyd_forms_required_fields_example', 10, 3 );

/**
 * Add custom error messages for form submissions, after validation.
 * Eg. Add custom error messages for the custom field 'my_custom_field' for a specific form.
 *
 * @param array $errors    Current form errors.
 * @param array $form_data Validated form data.
 * @param int   $form_id   WP_Post ID of the form.
 *
 * @return array $errors
 */
function greyd_forms_handler_errors_example( $errors, $form_data, $form_id ) {

	// Get the form post object
	$form = get_post( $form_id );

	// Check if the form is the one we want to add custom error messages for
	if ( $form && $form->post_name === 'my-contact-form' ) {

		// Check the custom field 'my_custom_field' and perform custom validation
		$my_custom_field_value = isset( $form_data['my_custom_field'] ) ? $form_data['my_custom_field'] : null;

		if ( ! $my_custom_field_value ) {
			$errors['my_custom_field'] = 'Please enter a value for the custom field.';
		}
		else if ( strlen( $my_custom_field_value ) < 5 ) {
			$errors['my_custom_field'] = 'The custom field must be at least 5 characters long.';
		}
	}

	return $errors;
}
add_filter( 'greyd_forms_handler_errors', 'greyd_forms_handler_errors_example', 10, 3 );

/**
 * Filter the validated form data before submission.
 * Eg. modify the validated form data before it is saved to the entry.
 *
 * @param array $form_data The validated form data.
 * @param int   $form_id   The WP_Post ID of the form being submitted.
 *
 * @return array The validated form data.
 */
function greyd_forms_filter_validated_data_example( $form_data, $form_id ) {
	
	// Get the form post object
	$form = get_post( $form_id );

	// Check if the form is the one we want to add custom data for
	if ( $form && $form->post_name === 'my-contact-form' ) {

		// Check the custom field 'my_custom_field' and perform custom validation
		$my_custom_field_value = isset( $form_data['my_custom_field'] ) ? $form_data['my_custom_field'] : null;

		if ( $my_custom_field_value ) {
			$form_data['my_custom_field'] = sanitize_text_field( $my_custom_field_value ) . ' (custom data added)';
		}
	}

	return $form_data;
}
add_filter( 'greyd_forms_validated_data', 'greyd_forms_filter_validated_data_example', 10, 3 );

/**
 * Perform an action before the form entry is saved.
 * Eg. Log a custom option value to the console log in the frontend.
 * 
 * @param array $form_data The validated form data.
 * @param int   $form_id   The WP_Post ID of the form being submitted.
 */
function greyd_forms_action_before_entry_example( $form_data, $form_id ) {

	// Get the form post object
	$form = get_post( $form_id );

	// Check if the form is the one we want to log data for
	if ( $form && $form->post_name === 'my-contact-form' ) {

		$option_value = get_option( 'my_custom_option' );

		/**
		 * Use the action 'formhandler_log' to log the custom option value to the
		 * console log in the frontend. This is useful for debugging purposes.
		 * 
		 * @param mixed  $option_value The custom option value to log.
		 * @param string $description  A description of the custom option value.
		 */
		do_action( 'formhandler_log', $option_value, 'Custom option value' );

		if ( $option_value === 'prevent_submission' ) {
			
			/**
			 * Use the action 'formhandler_error' to prevent the form submission.
			 * This aborts the form submission and displays an error message to the user in
			 * the frontend. This is useful for preventing form submissions based on custom
			 * conditions.
			 */
			do_action( 'formhandler_error', 'The form submission was prevented due to a custom option value.' );
		}
	}
}
add_action( 'greyd_forms_action_before_entry', 'greyd_forms_action_before_entry_example', 10, 2 );

/**
 * Perform an action after the form entry is saved, but before the default
 * mails like the Double Opt-In mail are sent. This action will therefore 
 * fire even before the users email address is confirmed by clicking the
 * confirmation link.
 * 
 * There are alternative action hooks available after specific events:
 * 
 * @action greyd_forms_action_after_mails
 * If you want to instead perform an action after the default mails are sent,
 * use the 'greyd_forms_action_after_mails' action hook. It takes the same
 * arguments as this action hook, but will fire only fire once the default
 * mails have been sent.
 * 
 * @action greyd_forms_action_after_optin
 * If you want to perform an action after the user has confirmed their email
 * address by clicking the confirmation link, use the 'greyd_forms_action_after_optin'
 * action hook. It takes the same arguments as this action hook, but will fire
 * only after the user has confirmed their email address. This has no effect
 * if the Double Opt-In feature is disabled.
 * 
 * @param array $form_data The validated form data.
 * @param int   $form_id   The WP_Post ID of the form being submitted.
 * @param int   $entry_id  The WP_Post ID of the form entry.
 */
function greyd_forms_action_after_entry_example( $form_data, $form_id, $entry_id ) {

	// Get the form post object
	$form = get_post( $form_id );

	// Check if the form is the one we want to send a custom email notification for
	if ( $form && $form->post_name === 'my-contact-form' ) {

		// Get the custom field 'my_custom_field' value
		$my_custom_field_value = isset( $form_data['my_custom_field'] ) ? $form_data['my_custom_field'] : '';

		// Send the data to an API
		$response = wp_remote_post( 'https://api.example.com/submit', array(
			'body' => array(
				'form_data'    => json_encode( $form_data ),
			),
		) );

		/**
		 * Use the action 'formhandler_log' to log the API response to the console log
		 * in the frontend. This is useful for debugging purposes.
		 */
		do_action( 'formhandler_log', $response, 'API response' );

		if ( is_wp_error( $response ) ) {
			
			/**
			 * Use the action 'formhandler_error' to display an error message to the user in
			 * the frontend. This is useful for displaying error messages when sending data to
			 * an external API fails.
			 */
			do_action( 'formhandler_error', 'An error occurred while sending the data to the API.' );
		}
		else {
			
			/**
			 * Use the action 'formhandler_success' to display a success message to the user in
			 * the frontend.
			 * 
			 * @param array $args An array of arguments to pass to the action. Currently supports:
			 *                 - post_id: The WP_Post ID of the form being submitted.
			 *                 - message: The success message to display.
			 */
			do_action( 'formhandler_success', array(
				'post_id' => $form_id,
				'message' => 'Thank you for your submission. Your data has been sent to the API.',
			) );
		}
	}
}
add_action( 'greyd_forms_action_after_entry', 'greyd_forms_action_after_entry_example', 10, 3 );

/**
 * Filter the mail content for the admin notification mail.
 * 
 * There are a lot more hooks available for modifying different attributes of
 * all mails sent by the plugin. For example, you can modify the mail subject,
 * the mail headers, the mail recipients etc. Take a look at the plugin code
 * for more information (greyd_tp_forms/inc/handler.php). Here is a brief list
 * of the most important hooks:
 *     @filter greyd_forms_admin_mail_to
 *     @filter greyd_forms_admin_mail_from
 *     @filter greyd_forms_admin_mail_subject
 *     @filter greyd_forms_admin_mail_content
 *     @filter greyd_forms_admin_mail_headers
 *   User Opt-In mails:
 *     @filter greyd_forms_optin_mail_to
 *     @filter greyd_forms_optin_mail_from
 *     @filter greyd_forms_optin_mail_subject
 *     @filter greyd_forms_optin_mail_content
 *     @filter greyd_forms_optin_mail_headers
 *   User registration mails (after Opt-In):
 *     @filter greyd_forms_registration_mail_to
 *     @filter greyd_forms_registration_mail_from
 *     @filter greyd_forms_registration_mail_subject
 *     @filter greyd_forms_registration_mail_content
 *     @filter greyd_forms_registration_mail_headers
 * 
 *
 * @param string $content   Full mail HTML content.
 * @param int    $form_id   WP_Post ID of the form.
 * @param array  $form_data Validated form data.
 * 
 * @return string The modified mail content.
 */
function greyd_forms_admin_mail_content_example( $content, $form_id, $form_data ) {

	// Get the form post object
	$form = get_post( $form_id );

	// Check if the form is the one we want to modify the mail content for
	if ( $form && $form->post_name === 'my-contact-form' ) {

		// Get the custom field 'my_custom_field' value
		$my_custom_field_value = isset( $form_data['my_custom_field'] ) ? $form_data['my_custom_field'] : '';

		// Append the custom field value to the mail content
		$content .= '<p><strong>Custom Field:</strong> ' . esc_html( $my_custom_field_value ) . '</p>';
	}

	return $content;
}
add_action( 'greyd_forms_admin_mail_content', 'greyd_forms_admin_mail_content_example', 10, 3 );